/*
 * Copyright 2017 Maxst, Inc. All Rights Reserved.
 */

package com.maxst.ar.sample.vivarTracker;

import android.content.res.Configuration;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import com.maxst.ar.CameraDevice;
import com.maxst.ar.MaxstAR;
import com.maxst.ar.TrackerManager;
import com.maxst.ar.sample.ARActivity;
import com.maxst.ar.sample.R;
import com.maxst.ar.sample.util.TrackerResultListener;

public class ARPointTrackerActivity extends ARActivity implements View.OnClickListener, View.OnTouchListener {

	private ARPointTrackerRenderer arPointTargetRenderer;
	private GLSurfaceView glSurfaceView;
	private int anchorCount = 0;

	private String anchorName = "";

	private float[] lastPose = new float[16];

	private String getAnchorName(boolean next)
	{
		if (next)
			anchorCount++;

		return "anchor" + anchorCount;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_vivar_tracker);

		arPointTargetRenderer = new ARPointTrackerRenderer(this);
		glSurfaceView = (GLSurfaceView) findViewById(R.id.gl_surface_view);
		glSurfaceView.setEGLContextClientVersion(2);
		glSurfaceView.setRenderer(arPointTargetRenderer);
		glSurfaceView.setOnTouchListener(this);

		findViewById(R.id.btn_find).setOnClickListener(this);

		arPointTargetRenderer.listener = resultListener;
	}

	@Override
	protected void onResume() {
		super.onResume();

		glSurfaceView.onResume();

		CameraDevice.getInstance().start(0, 1280, 720);
		TrackerManager.getInstance().startTracker(0x1000);

		MaxstAR.onResume();
	}

	@Override
	protected void onPause() {
		super.onPause();

		glSurfaceView.onPause();
		CameraDevice.getInstance().stop();
		TrackerManager.getInstance().stopTracker();
		MaxstAR.onPause();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
		arPointTargetRenderer.setOrientation(newConfig.orientation);
	}

	@Override
	public boolean onTouch(View v, final MotionEvent event) {
		float x = event.getX();
		float y = event.getY();
		switch (event.getAction()) {
			case MotionEvent.ACTION_DOWN: {
				Log.i("ARPointTrackerActivity", "touch action down");
				System.arraycopy(arPointTargetRenderer.lastPose, 0, lastPose, 0, 16);
				break;
			}
			case MotionEvent.ACTION_UP: {
				Log.i("ARPointTrackerActivity", "touch action up");
				Point3f ncoord = arPointTargetRenderer.screenToLocal(x, y);
				anchorName = getAnchorName(true);
				String strPose = "";
				for(int i=0; i<15; i++)
					strPose += lastPose[i] + ",";
				strPose += lastPose[15];
				String addCommand = "{\"vivar\":\"add_prev_anchor_normalized\",\"name\":\"" + anchorName + "\",\"x_pos\":" + ncoord.x + ",\"y_pos\":" + ncoord.y +
						",\"search_pose\":\"" + strPose + "\",\"z_offset\":0.2}";
				TrackerManager.getInstance().addTrackerData(addCommand, false);
				Log.i("ARPointTrackerActivity", "add_prev_anchor_normalized command : " + addCommand);
				break;
			}
		}
		return true;
	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.btn_find:
				TrackerManager.getInstance().findSurface();
				break;
		}
	}

	private TrackerResultListener resultListener = new TrackerResultListener() {
		@Override
		public void sendData(final String metaData) {
			(ARPointTrackerActivity.this).runOnUiThread(new Runnable(){
				@Override
				public void run() {
				}
			});
		}

		@Override
		public void sendFusionState(final int state) {
			(ARPointTrackerActivity.this).runOnUiThread(new Runnable(){
				@Override
				public void run() {
				}
			});
		}
	};
}
