/*
 * Copyright 2017 Maxst, Inc. All Rights Reserved.
 */
package com.maxst.ar.sample.vivarTracker;

import android.app.Activity;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView.Renderer;
import android.util.Log;

import com.maxst.ar.CameraDevice;
import com.maxst.ar.GuideInfo;
import com.maxst.ar.MaxstAR;
import com.maxst.ar.MaxstARUtil;
import com.maxst.ar.Trackable;
import com.maxst.ar.TrackedImage;
import com.maxst.ar.TrackerManager;
import com.maxst.ar.TrackingResult;
import com.maxst.ar.TrackingState;
import com.maxst.ar.sample.arobject.BackgroundRenderHelper;
import com.maxst.ar.sample.arobject.FeaturePointRenderer;
import com.maxst.ar.sample.arobject.TexturedCube;
import com.maxst.ar.sample.util.TrackerResultListener;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;


class ARPointTrackerRenderer implements Renderer {

	public static final String TAG = ARPointTrackerRenderer.class.getSimpleName();

	private TexturedCube texturedCube;
	private FeaturePointRenderer featurePointRenderer;

	private int surfaceWidth;
	private int surfaceHeight;
	private int camWidth = 1280;
	private int camHeight = 720;

	private int orientation;
	private final Activity activity;
	private BackgroundRenderHelper backgroundRenderHelper;
	public TrackerResultListener listener = null;

	public float[] lastPose = new float[16];

	ARPointTrackerRenderer(Activity activity) {
		this.activity = activity;
	}

	public Point3f screenToLocal(float screenX, float screenY) {
		// calculate normalized screen coordiante
		float sw = surfaceWidth;
		float sh = surfaceHeight;

		float cw = camWidth;
		float ch = camHeight;

		float nw = (sw > sh) ? sw : sh;
		float nh = (sw > sh) ? sh : sw;

		float px = (sw > sh) ? screenX : screenY;
		float py = (sw > sh) ? screenY : nh - screenX;

		float rw = nw / cw;
		float rh = nh / ch;

		float margin_nh = 0.f;
		float margin_nw = 0.f;

		// cut height
		if (rw > rh) {
			float vh = rw * ch;
			margin_nh = (vh-nh) * 0.5f;
			py += margin_nh;
			Log.i(TAG, "margin h = " + margin_nh);
		} else {
			float vw = rh * cw;
			margin_nw = (vw-nw) * 0.5f;
			px += margin_nw;
			Log.i(TAG, "margin w = " + margin_nw);
		}

		float nx = (px - (nw * 0.5f + margin_nw)) / nw;
		float ny = (py - (nh * 0.5f + margin_nh)) / nw;

		return new Point3f(nx, ny, 1.0f);
	}

	@Override
	public void onSurfaceCreated(GL10 unused, EGLConfig config) {
		GLES20.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

		Bitmap bitmap = MaxstARUtil.getBitmapFromAsset("subtraction.png", activity.getAssets());
		texturedCube = new TexturedCube();
		texturedCube.setTextureBitmap(bitmap);
		featurePointRenderer = new FeaturePointRenderer();
		Bitmap blueBitmap = MaxstARUtil.getBitmapFromAsset("bluedot.png", activity.getAssets());
		Bitmap redBitmap = MaxstARUtil.getBitmapFromAsset("reddot.png", activity.getAssets());
		featurePointRenderer.setFeatureImage(blueBitmap, redBitmap);
		backgroundRenderHelper = new BackgroundRenderHelper();

		CameraDevice.getInstance().setClippingPlane(0.03f, 70.0f);
	}

	@Override
	public void onSurfaceChanged(GL10 unused, int width, int height) {
		surfaceWidth = width;
		surfaceHeight = height;

		if (orientation == Configuration.ORIENTATION_PORTRAIT) {
			Log.i(TAG, "portrait");
			texturedCube.setRotation(0, 1, 1, 1);
		} else {
			texturedCube.setRotation(-90, 0, 0, 1);
		}
		texturedCube.setScale(0.2f, 0.2f, 0.01f);

		MaxstAR.onSurfaceChanged(width, height);
	}

	void setOrientation(int orientation) {
		this.orientation = orientation;
	}

	private void printMatrix4x4(String matrixName, float[] m) {
		Log.d("vivar", "Vivar Fusion :: " + matrixName + "\n" +
				m[0] + ", " + m[4] + ", " + m[8] + ", " + m[12] + ", \n" +
				m[1] + ", " + m[5] + ", " + m[9] + ", " + m[13] + ", \n" +
				m[2] + ", " + m[6] + ", " + m[10] + ", " + m[14] + ", \n" +
				m[3] + ", " + m[7] + ", " + m[11] + ", " + m[15] + ", \n");
	}

	private void multiplyPose(float[] dst, float[] src1, float[] src2)
	{
		float[][] src1_ = {{src1[0], src1[4], src1[8], src1[12]},
				{src1[1], src1[5], src1[9], src1[13]},
				{src1[2], src1[6], src1[10], src1[14]},
				{src1[3], src1[7], src1[11], src1[15]}};
		float[][] src2_ = {{src2[0], src2[4], src2[8], src2[12]},
				{src2[1], src2[5], src2[9], src2[13]},
				{src2[2], src2[6], src2[10], src2[14]},
				{src2[3], src2[7], src2[11], src2[15]}};

		for(int i=0; i<4; i++)
		{
			for(int j=0; j<4; j++)
			{
				float sum = 0.0f;
				for(int k=0; k<4; k++)
					sum += src1_[i][k] * src2_[k][j];

				dst[j*4+i] = sum;
			}
		}
	}

	@Override
	public void onDrawFrame(GL10 unused) {
		GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT | GLES20.GL_DEPTH_BUFFER_BIT);
		GLES20.glViewport(0, 0, surfaceWidth, surfaceHeight);

		TrackingState state = TrackerManager.getInstance().updateTrackingState();
		TrackingResult trackingResult = state.getTrackingResult();

		TrackedImage image = state.getImage();
		float[] projectionMatrix = CameraDevice.getInstance().getProjectionMatrix();
		float[] backgroundPlaneInfo = CameraDevice.getInstance().getBackgroundPlaneInfo();

		backgroundRenderHelper.drawBackground(image, projectionMatrix, backgroundPlaneInfo);
		featurePointRenderer.setProjectionMatrix(projectionMatrix);
		GuideInfo gi = TrackerManager.getInstance().getGuideInformation();
		featurePointRenderer.draw(gi, trackingResult);
		int trackingCount = trackingResult.getCount();

		if(trackingCount == 0) {
			return;
		}

		GLES20.glEnable(GLES20.GL_DEPTH_TEST);
		float[] basePose = new float[16];
		for (int i = 0; i < trackingCount; i++) {
			Trackable trackable = trackingResult.getTrackable(i);
			listener.sendData(trackable.getName());

			if (trackable.getName().compareTo("status") == 0) {
				continue;
			} else if (trackable.getName().compareTo("default") == 0) {
				System.arraycopy(trackable.getPoseMatrix(), 0, basePose, 0, 16);
				System.arraycopy(trackable.getPoseMatrix(), 0, lastPose, 0, 16);
				continue;
			}

			float[] pose = new float[16];
			System.arraycopy(trackable.getPoseMatrix(), 0, pose, 0, 16);

			float[] eachPose = new float[16];
			multiplyPose(eachPose, basePose, pose);

			texturedCube.setProjectionMatrix(projectionMatrix);
			texturedCube.setTransform(eachPose);
			texturedCube.draw();
		}
	}
}
